package main

import (
	"errors"
	"fmt"
	"github.com/tursom/checksum/checksum"
	"io"
	"io/fs"
	"os"
	"path/filepath"
)

var (
	hashFuncs = []func() checksum.Func{
		func() checksum.Func { return &checksum.Md5{} },
		func() checksum.Func { return &checksum.Sha1{} },
		func() checksum.Func { return &checksum.Sha256{} },
		func() checksum.Func { return &checksum.Sha512{} },
		func() checksum.Func { return &checksum.Murmur332{} },
		func() checksum.Func { return &checksum.Murmur364{} },
		func() checksum.Func { return &checksum.Murmur3128{} },
		func() checksum.Func { return &checksum.Adler32{} },
		func() checksum.Func { return &checksum.Crc32Ieee{} },
		func() checksum.Func { return &checksum.Crc32Castagnoli{} },
		func() checksum.Func { return &checksum.Crc32Koopman{} },
		func() checksum.Func { return &checksum.Crc64Iso{} },
		func() checksum.Func { return &checksum.Crc64Ecma{} },
		func() checksum.Func { return &checksum.Fnv32{} },
		func() checksum.Func { return &checksum.Fnv32a{} },
		func() checksum.Func { return &checksum.Fnv64{} },
		func() checksum.Func { return &checksum.Fnv64a{} },
		func() checksum.Func { return &checksum.Fnv128{} },
		func() checksum.Func { return &checksum.Fnv128a{} },
	}
)

func main() {
	path, checksumType := readConfig()
	doCheck(path, hashFuncs[checksumType-1])
	waitExit()
}

func readConfig() (string, uint) {
	fmt.Println("Please select checksum path:")
	fmt.Print(">>> ")
	var path string
	if _, err := fmt.Scanln(&path); err != nil {
		panic(err)
	}

	fmt.Println("Please select checksum type:")
	for i, f := range hashFuncs {
		fmt.Printf("%d. %s\n", i+1, f().String())
	}
	fmt.Print(">>> ")
	var checksumType uint
	if _, err := fmt.Scanln(&checksumType); err != nil {
		panic(err)
	}
	return path, checksumType
}

func doCheck(path string, factory func() checksum.Func) {
	checkFile(path, factory)
}

func checkFile(path string, factory func() checksum.Func) {
	file, err := os.Open(path)
	if err != nil {
		panic(err)
	}
	defer file.Close()

	fileStat, err := file.Stat()
	if err != nil {
		panic(errors.New(fmt.Sprintf("failed to check status: %s", err)))
	}

	switch mode := fileStat.Mode(); {
	case mode.IsDir():
		filepath.Walk(file.Name(), func(subPath string, info fs.FileInfo, err error) error {
			if subPath == path {
				return nil
			}
			checkFile(subPath, factory)
			return nil
		})
	case mode.IsRegular():
		f := factory()

		buffer := make([]byte, 1024)
		for {
			n, err := file.Read(buffer)
			if n == 0 || err == io.EOF {
				break
			} else if err != nil {
				panic(err)
			}
			f.Append(buffer[0:n])
		}

		fmt.Printf("file: %s\nchecksum: %x\n\n", path, f.Finish())
	}
}

func waitExit() {
	fmt.Println("Press enter to exit.")
	_, _ = fmt.Scanln()
}
